import json
from pathlib import Path
from typing import Any, Optional

from ninja import NinjaAPI
from ninja.openapi.docs import HttpRequest, HttpResponse, Swagger, _csrf_needed, _render_cdn_template
from ninja.types import DictStrAny

from ninja_oauth2.openapi.utils import get_oauth2_redirect_url

ABS_TPL_PATH = Path(__file__).parent.parent / "templates/ninja_oauth2/"


class SwaggerOAuth2(Swagger):
    template_cdn = str(ABS_TPL_PATH / "swagger_cdn.html")
    default_settings = {
        "layout": "BaseLayout",
        "deepLinking": True,
    }

    def __init__(self, settings: Optional[DictStrAny] = None, auth: Optional[DictStrAny] = None):
        self.auth = auth
        super().__init__(settings)

    def render_page(self, request: HttpRequest, api: "NinjaAPI", **kwargs: Any) -> HttpResponse:
        self.settings["url"] = self.get_openapi_url(api, kwargs)

        if self.auth:
            self.settings["oauth2RedirectUrl"] = get_oauth2_redirect_url(api.docs_url)

        context = {
            "swagger_settings": json.dumps(self.settings, indent=1),
            "api": api,
            "add_csrf": _csrf_needed(api),
            "add_auth": bool(self.auth),
            "swagger_auth": json.dumps(self.auth, indent=1),
        }
        return _render_cdn_template(request, self.template_cdn, context)



from typing import Any, Optional

import jwt
import requests
from ninja.errors import HttpError
from ninja.openapi.docs import HttpRequest
from ninja.security.base import AuthBase


class OAuth2AuthorizationCodeBearer(AuthBase):
    openapi_type = "oauth2"

    def __init__(self, authorization_url: str, token_url: str, public_key_url: str, auto_error: bool = True) -> None:
        self.public_key_url = public_key_url
        self.auto_error = auto_error
        self.public_key = self._get_public_key()
        self.openapi_flows = {
            "authorizationCode": {"scopes": {}, "authorizationUrl": authorization_url, "tokenUrl": token_url}
        }

        super().__init__()

    def __call__(self, request: HttpRequest) -> Optional[Any]:
        authorization = request.headers.get("Authorization")

        if not authorization:
            if self.auto_error:
                raise HttpError(403, "Not authenticated")
            else:
                return None

        parts = authorization.split(" ")

        if len(parts) != 2 or parts[0].lower() != "bearer":
            if self.auto_error:
                raise HttpError(403, "Not authenticated")
            else:
                return None

        token = parts[1]

        try:
            token_info = jwt.decode(token, self.public_key, audience="account", algorithms=["RS256"])
        except Exception as e:
            if self.auto_error:
                raise HttpError(401, "Invalid token") from e
            else:
                return None

        return self.authenticate(request, token_info)

    def _get_public_key(self) -> str:
        try:
            response = requests.get(self.public_key_url)
            response.raise_for_status()

            content = response.json()
            public_key = content.get("public_key")

            if public_key:
                return f"-----BEGIN PUBLIC KEY-----\n{public_key}\n-----END PUBLIC KEY-----"
            else:
                raise ValueError("Public key not found in the response.")

        except requests.RequestException as e:
            raise Exception(f"Get public key failed with the following error: {e}.") from e

    def authenticate(self, request: HttpRequest, token_info: dict) -> Optional[Any]:
        return token_info



from functools import partial
from typing import TYPE_CHECKING, Any, List

from django.urls import path
from ninja.openapi.views import openapi_json, openapi_view

from ninja_oauth2.openapi.utils import get_oauth2_redirect_url
from ninja_oauth2.openapi.views import oauth2_redirect_view

if TYPE_CHECKING:
    from ninja import NinjaAPI  # pragma: no cover


def get_openapi_urls(api: "NinjaAPI") -> List[Any]:
    result = []

    if api.openapi_url:
        view = partial(openapi_json, api=api)
        if api.docs_decorator:
            view = api.docs_decorator(view)  # type: ignore
        result.append(
            path(api.openapi_url.lstrip("/"), view, name="openapi-json"),
        )

        assert api.openapi_url != api.docs_url, "Please use different urls for openapi_url and docs_url"

        if api.docs_url:
            view = partial(openapi_view, api=api)
            if api.docs_decorator:
                view = api.docs_decorator(view)  # type: ignore
            result.append(
                path(api.docs_url.lstrip("/"), view, name="openapi-view"),
            )

            redirect_view = partial(oauth2_redirect_view, api=api)
            redirect_url = get_oauth2_redirect_url(api.docs_url)
            result.append(
                path(redirect_url, redirect_view, name="openapi-redirect-view"),
            )

    return result



from pathlib import Path
from typing import TYPE_CHECKING, Any

from django.http import HttpRequest, HttpResponse
from ninja.openapi.docs import _render_cdn_template

if TYPE_CHECKING:
    from ninja import NinjaAPI  # pragma: no cover

ABS_TPL_PATH = Path(__file__).parent.parent / "templates/ninja_oauth2/"


def oauth2_redirect_view(request: HttpRequest, api: "NinjaAPI", **kwargs: Any) -> HttpResponse:
    template_cdn = str(ABS_TPL_PATH / "oauth2-redirect.html")

    return _render_cdn_template(request, template_cdn)



from typing import List, Union

from django.urls import URLPattern, URLResolver
from ninja import NinjaAPI
from ninja.openapi.urls import get_root_url

from ninja_oauth2.openapi.urls import get_openapi_urls


class NinjaAPIOAuth2(NinjaAPI):
    def _get_urls(self) -> List[Union[URLResolver, URLPattern]]:
        result = get_openapi_urls(self)

        for prefix, router in self._routers:
            result.extend(router.urls_paths(prefix))

        result.append(get_root_url(self))
        return result



"""Django Ninja - Fast Django REST framework"""

__version__ = "1.1.0"

from ninja_oauth2.main import NinjaAPIOAuth2
from ninja_oauth2.openapi.docs import SwaggerOAuth2

__all__ = ["NinjaAPIOAuth2", "SwaggerOAuth2"]
